﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using PermutationAlgorithms.PermAlgorithms;
using PermutationAlgorithms.PermAlgorithms.Impl;
using System.Diagnostics;

namespace PermutationAlgorithms
{
    class PermAlgoTests : IPermutationVisitor
    {
        private HashSet<string> permutationSet;
        private ArgOptions options;

        public PermAlgoTests(ArgOptions options)
        {
            this.options = options;
            permutationSet = new HashSet<string>();
        }

        /// <summary>
        /// Exeucte the permutation generation.
        /// </summary>
        public void Run()
        {
            /* instantiate the correct permutation algorithm class from given algorithm enum.
             * Here it's IMPORTANT that the name of the enum exactly matches the class name */

            Type enumAlgoType = typeof(PermutationAlgorithms.EAlgorithms);
            string algoname = Enum.GetName(enumAlgoType, options.Algorithm);

            Console.Out.WriteLine("Selected algorithm: " + algoname);
            Stopwatch measureTimer = new Stopwatch();
            measureTimer.Start();

            Type classType = Type.GetType("PermutationAlgorithms.PermAlgorithms.Impl." + algoname, true);
            Object[] args = { options.NumOfItems, this };
            object classInstance = Activator.CreateInstance(classType, args);
            AbstractPermAlgo algo = (AbstractPermAlgo)classInstance;

            RunPermAlgo(algo);
            measureTimer.Stop();

            Console.Out.WriteLine("Elapsed time: " + measureTimer.ElapsedMilliseconds + "ms");
        }

        /// <summary>
        /// Executes the given permutation algorithm
        /// </summary>
        /// <param name="algo"></param>
        public void RunPermAlgo(AbstractPermAlgo algo)
        {
            algo.Generate();

            Console.Out.WriteLine("#Permutations: " + algo.PermutationCounter);
            long expNumPermutations = algo.NumberOfPermutations();

            if (algo.PermutationCounter != expNumPermutations)
            {
                Console.Out.WriteLine(" -> ERROR: Expected #permutations = " + expNumPermutations);
            }

            Console.Out.WriteLine("#Swaps: " + algo.SwapCounter);
        }

        /// <summary>
        /// Called each time a new permutation was found.
        /// Checks that permutation was not already reported before and has correct size.
        /// </summary>
        /// <param name="permutatedItems"></param>
        public void VisitPermutation(System.Collections.ObjectModel.ReadOnlyCollection<char> permutatedItems)
        {
            StringBuilder sb = new StringBuilder();
            foreach (char c in permutatedItems)
            {
                sb.Append(c);
                sb.Append(" ");
            }

            if (!options.SuppressPrint)
            {
                Console.Out.WriteLine(sb.ToString());

                /* check if permutation has not already been generated */
                if (!permutationSet.Add(sb.ToString()))
                {
                    Console.Out.WriteLine(" -> ERROR: Permutation " + sb.ToString() + " already exists");
                }

                /* check that permutation has correct size */
                if (sb.Length != options.NumOfItems * 2)
                {
                    Console.Out.WriteLine(" -> ERROR: Permutation " + sb.ToString() + " has not size 8.");
                }

            }
        }

    }
}
