﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

namespace PermutationAlgorithms.PermAlgorithms
{
    abstract class AbstractPermAlgo
    {
        protected static readonly char StartItem = 'A';

        /// <summary>
        /// Permutation visitor reference. Used to indicate new calculated permuation.
        /// </summary>
        private IPermutationVisitor permVisitor;

        /// <summary>
        /// array holding the items to permute
        /// </summary>
        protected char[] items;

        /****************************************************
         * Properties
         ****************************************************/

        /// <summary>
        /// Number of indicated permutations.
        /// </summary>
        public long PermutationCounter { get; protected set; }


        public int ItemCount { get; protected set; }

        public int SwapCounter { get; private set; }


        /// <summary>
        /// Constructor.
        /// </summary>
        /// <param name="numOfItems">number of items in array to permutate</param>
        /// <param name="permVisitor"></param>
        public AbstractPermAlgo(int numOfItems, IPermutationVisitor permVisitor)
        {
            // input parameter validation
            if ((numOfItems < 1) || (numOfItems > 16))
                throw new ArgumentException("Invalid numOfItems value: Valid range [1,18], given " + numOfItems);

            if (permVisitor == null)
                throw new ArgumentNullException("permVisitor");

            // init items
            items = new char[numOfItems];
            ItemCount = numOfItems;
            InitItemSet();
            
            SwapCounter = 0;

            PermutationCounter = 0;
            this.permVisitor = permVisitor;
        }

        /// <summary>
        /// Generates all possible permutations of the set.
        /// </summary>
        abstract public void Generate();

        /// <summary>
        /// Initializes the item set in ascending order.
        /// </summary>
        protected void InitItemSet()
        {
            for (int i = 0; i < ItemCount; i++)
            {
                items[i] = (char)((int)StartItem + i);
            }
        }

        protected void VisitPermutation()
        {
            PermutationCounter++;
            permVisitor.VisitPermutation(Array.AsReadOnly(items));
        }

        protected void Swap(ref char val1, ref char val2)
        {
            char tmp = val1;
            val1 = val2;
            val2 = tmp;

            SwapCounter++;
        }

        public static long Factorial(long n)
        {
            int result = 1;
            for (int i = 1; i <= n; i++)
            {
                result *= i;
            }
            return result;
        }

        public long NumberOfPermutations()
        {
            return Factorial(ItemCount);
        }
    }
}
