﻿/*
 * Implementations of different permutation algorithms.
 * 
 * Implementations variants:
 *  - Recursive Backtracking
 *  - Recursive Backtracking (direction reversed)
 *  - Heap's algorithm (recursive)
 *  - Heap's algorithm (iterative)
 *  - Algorithm L
 *  - Algorithm P
 *  - Algorithm T
 *  - K-Level Algorithm recursive
 *
 * Sunshine, September 2016
 *
 * www.sunshine2k.de || www.bastian-molkenthin.de
 */
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.IO;
using PermutationAlgorithms.PermAlgorithms;

namespace PermutationAlgorithms
{

    public enum EAlgorithms
    {
        BackTrackRecursive = 0,
        AlgorithmL = 1,
        HeapRecursive = 2,
        HeapIterative = 3,
        AlgorithmP = 4,
        AlgorithmT = 5,
        BackTrackRecursiveExchangeReversed = 6,
        KLevelRecursive = 7
    }

    public class ArgOptions
    {
        public EAlgorithms Algorithm { get; private set; }
        public int NumOfItems { get; private set; }
        public bool SuppressPrint { get; private set; }

        public ArgOptions(int numOfItems, EAlgorithms algorithm, bool suppressPrint)
        {
            this.NumOfItems = numOfItems;
            this.Algorithm = algorithm;
            this.SuppressPrint = suppressPrint;
        }
    }

    class Program
    {
        public static readonly int MAX_ITEMS = 16;

        static void PrintUsage()
        {
            Console.ForegroundColor = ConsoleColor.White;
            using (TextWriter textOut = Console.Out)
            {
                textOut.WriteLine("Sunshine's example implementations of permutation algorithms");
                textOut.WriteLine("www.sunshine2k.de | www.bastian-molkenthin.de");
                Console.ResetColor();

                textOut.WriteLine();
                textOut.WriteLine("Synopsis:");
                textOut.WriteLine("PermutationAlgorithms.exe <numOfItems> <algorithm>");

                textOut.WriteLine("<numOfItems>: Number of items in set, range [1, " + MAX_ITEMS + "]");

                /* print all available algorithms from enumeration */
                textOut.WriteLine("<algorithm>: Algorithm to use");
                Type algoEnumType = typeof(PermutationAlgorithms.EAlgorithms);
                string[] algoNames = Enum.GetNames(algoEnumType);
                int[] algoValues = (int[])Enum.GetValues(algoEnumType);
                for (int i = 0; i < algoNames.Length; i++)
                {
                    textOut.WriteLine(" " + algoValues[i] + ": " + algoNames[i]);
                }

                textOut.WriteLine();
                textOut.WriteLine("Examples: PermutationAlgorithms 4 3");
            }
        }

        static void Main(string[] args)
        {
            ArgOptions options;

            if (args.Length < 2)
            {
                PrintUsage();
                return;
            }

            if (!ParseCommandLine(args, out options))
                return;

            PermAlgoTests permAlgos = new PermAlgoTests(options);

            permAlgos.Run();

#if DEBUG
            Console.In.Read();
#endif
        }

        private static bool ParseCommandLine(string[] args, out ArgOptions options)
        {
            int numOfItems;
            EAlgorithms algorithm;
            bool supressPrint = false;
            options = null;

            /* first element are the number of items in the set */
            if (!int.TryParse(args[0], out numOfItems))
            {
                Console.Out.WriteLine("Invalid parameter for number of items: " + args[0]);
                return false;
            }
            if (numOfItems <= 0 || numOfItems > 16)
            {
                Console.Out.WriteLine("Number of items out of valid range [1, 16]");
                return false;
            }
            
            /* second element is the algorithm to be used */
            if (!Enum.TryParse<EAlgorithms>(args[1], out algorithm) || !(Enum.IsDefined(typeof(PermutationAlgorithms.EAlgorithms), algorithm)))
            {
                Console.Out.WriteLine("Invalid parameter for algorithm: " + args[1]);
                return false;
            }

            if (args.Length > 2 && args[2].Equals("-noprint"))
            {
                supressPrint = true;
            }

            options = new ArgOptions(numOfItems, algorithm, supressPrint);
            return true;
        }
    }
}
